@@ -436,6 +436,7 @@ create_user (GisAccountPage *page)
   GisAccountPagePrivate *priv = page->priv;
   const gchar *username;
   const gchar *fullname;
+  const gchar *language;
   GError *error;
 
   username = gtk_combo_box_text_get_active_text (OBJ(GtkComboBoxText*, "account-username-combo"));
@@ -444,10 +445,19 @@ create_user (GisAccountPage *page)
   error = NULL;
 
   priv->act_user = act_user_manager_create_user (priv->act_client, username, fullname, priv->account_type, &error);
+  language = gis_driver_get_user_language (GIS_PAGE (page)->driver);
+  if (language)
+    act_user_set_language (priv->act_user, language);
+
   if (error != NULL) {
     g_warning ("Failed to create user: %s", error->message);
     g_error_free (error);
   }
+  /* TODO:We don't support coming back to this page to modify the values after
+   * the user has been created. For now just disable it
+   */
+  gtk_widget_set_sensitive (GTK_WIDGET (page), FALSE);
+  gtk_widget_set_sensitive (priv->action, FALSE);
 }
 
 static void save_account_data (GisAccountPage *page);
@@ -510,7 +520,7 @@ local_create_user (GisAccountPage *page)
   if (strlen (password) == 0)
     act_user_set_account_type (priv->act_user, ACT_USER_PASSWORD_MODE_NONE);
   else
-    act_user_set_password (priv->act_user, password, NULL);
+    act_user_set_password (priv->act_user, password, "");
 
   gis_driver_set_user_permissions (GIS_PAGE (page)->driver,
                                    priv->act_user,
