@@ -320,6 +320,16 @@ goa_account_removed (GoaClient *client, GoaObject *object, gpointer user_data)
   remove_account_from_list (page, object);
 }
 
+static void
+network_status_changed (GNetworkMonitor *monitor,
+                        gboolean         available,
+                        gpointer         user_data)
+{
+  GisGoaPage *page = GIS_GOA_PAGE (user_data);
+
+  gtk_widget_set_visible (GTK_WIDGET (page), available);
+}
+
 static void
 gis_goa_page_constructed (GObject *object)
 {
@@ -327,6 +337,7 @@ gis_goa_page_constructed (GObject *object)
   GisGoaPagePrivate *priv = page->priv;
   GtkWidget *button;
   GError *error = NULL;
+  GNetworkMonitor *network_monitor = g_network_monitor_get_default ();
 
   G_OBJECT_CLASS (gis_goa_page_parent_class)->constructed (object);
 
@@ -352,9 +363,13 @@ gis_goa_page_constructed (GObject *object)
   g_signal_connect (priv->goa_client, "account-removed",
                     G_CALLBACK (goa_account_removed), page);
 
+  g_signal_connect (network_monitor, "network-changed",
+                    G_CALLBACK (network_status_changed), page);
+
   gis_page_set_complete (GIS_PAGE (page), TRUE);
 
-  gtk_widget_show (GTK_WIDGET (page));
+  if (g_network_monitor_get_network_available (network_monitor))
+    gtk_widget_show (GTK_WIDGET (page));
 }
 
 static void
@@ -362,9 +377,12 @@ gis_goa_page_dispose (GObject *object)
 {
   GisGoaPage *page = GIS_GOA_PAGE (object);
   GisGoaPagePrivate *priv = page->priv;
+  GNetworkMonitor *network_monitor = g_network_monitor_get_default ();
 
   g_clear_object (&priv->goa_client);
 
+  g_signal_handlers_disconnect_by_func (network_monitor, G_CALLBACK (network_status_changed), page);
+
   G_OBJECT_CLASS (gis_goa_page_parent_class)->dispose (object);
 }
 
