@@ -16,7 +16,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  * Author: Sergey Udaltsov <svu@gnome.org>
+ *         Michael Wood <michael.g.wood@intel.com>
  *
+ * Based on gnome-control-center cc-region-panel.c
  */
 
 #define PAGE_ID "keyboard"
@@ -34,7 +36,6 @@
 #include "cc-input-chooser.h"
 
 #define GNOME_DESKTOP_USE_UNSTABLE_API
-#include <libgnome-desktop/gnome-languages.h>
 #include <libgnome-desktop/gnome-xkb-info.h>
 
 #ifdef HAVE_IBUS
@@ -51,9 +52,6 @@
 #define KEY_CURRENT_INPUT_SOURCE "current"
 #define KEY_INPUT_SOURCES        "sources"
 
-#define GNOME_SYSTEM_LOCALE_DIR "org.gnome.system.locale"
-#define KEY_REGION "region"
-
 #define INPUT_SOURCE_TYPE_XKB "xkb"
 #define INPUT_SOURCE_TYPE_IBUS "ibus"
 
@@ -61,46 +59,12 @@
 
 G_DEFINE_TYPE (GisKeyboardPage, gis_keyboard_page, GIS_TYPE_PAGE)
 
-#define WID(s) GTK_WIDGET (gtk_builder_get_object (self->priv->builder, s))
-
 #define KEYBOARD_PAGE_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GIS_TYPE_KEYBOARD_PAGE, GisKeyboardPagePrivate))
 
-typedef enum {
-        CHOOSE_LANGUAGE,
-        ADD_INPUT,
-        REMOVE_INPUT
-} SystemOp;
-
 struct _GisKeyboardPagePrivate {
-	GtkBuilder *builder;
-
-        GtkWidget   *login_button;
-        GtkWidget   *login_label;
-        gboolean     login;
-        GPermission *permission;
-        SystemOp     op;
         GDBusProxy  *localed;
-        GDBusProxy  *session;
         GCancellable *cancellable;
 
-        GtkWidget *overlay;
-        GtkWidget *notification;
-
-        GtkWidget *language_section;
-        GtkWidget *language_row;
-        GtkWidget *language_label;
-        GtkWidget *formats_row;
-        GtkWidget *formats_label;
-
-        ActUserManager *user_manager;
-        ActUser        *user;
-        GSettings      *locale_settings;
-
-        gchar *language;
-        gchar *region;
-        gchar *system_language;
-        gchar *system_region;
-
         GtkWidget *input_section;
         GtkWidget *input_list;
         GtkWidget *add_input;
@@ -119,6 +83,9 @@ struct _GisKeyboardPagePrivate {
 #endif
 };
 
+#define OBJ(type,name) ((type)gtk_builder_get_object(GIS_PAGE (self)->builder,(name)))
+#define WID(name) OBJ(GtkWidget*,name)
+
 static void
 gis_keyboard_page_finalize (GObject *object)
 {
@@ -128,21 +95,7 @@ gis_keyboard_page_finalize (GObject *object)
         g_cancellable_cancel (priv->cancellable);
         g_clear_object (&priv->cancellable);
 
-        if (priv->user_manager) {
-                g_signal_handlers_disconnect_by_data (priv->user_manager, self);
-                priv->user_manager = NULL;
-        }
-
-        if (priv->user) {
-                g_signal_handlers_disconnect_by_data (priv->user, self);
-                priv->user = NULL;
-        }
-
-        g_clear_object (&priv->permission);
         g_clear_object (&priv->localed);
-        g_clear_object (&priv->session);
-        g_clear_object (&priv->builder);
-        g_clear_object (&priv->locale_settings);
         g_clear_object (&priv->input_settings);
         g_clear_object (&priv->xkb_info);
 #ifdef HAVE_IBUS
@@ -152,14 +105,13 @@ gis_keyboard_page_finalize (GObject *object)
         g_clear_object (&priv->ibus_cancellable);
         g_clear_pointer (&priv->ibus_engines, g_hash_table_destroy);
 #endif
-        g_free (priv->language);
-        g_free (priv->region);
-        g_free (priv->system_language);
-        g_free (priv->system_region);
 
 	G_OBJECT_CLASS (gis_keyboard_page_parent_class)->finalize (object);
 }
 
+static void localed_proxy_ready (GObject *source, GAsyncResult *res, gpointer data);
+static void setup_input_section (GisKeyboardPage *self);
+
 static void
 gis_keyboard_page_constructed (GObject *object)
 {
@@ -168,8 +120,23 @@ gis_keyboard_page_constructed (GObject *object)
 
         G_OBJECT_CLASS (gis_keyboard_page_parent_class)->constructed (object);
 
-        gis_page_set_complete (GIS_PAGE (self), TRUE);
+        gtk_container_add (GTK_CONTAINER (self), WID ("keyboard_page"));
 
+        setup_input_section (self);
+
+        priv->cancellable = g_cancellable_new ();
+
+        g_dbus_proxy_new_for_bus (G_BUS_TYPE_SYSTEM,
+                                  G_DBUS_PROXY_FLAGS_GET_INVALIDATED_PROPERTIES,
+                                  NULL,
+                                  "org.freedesktop.locale1",
+                                  "/org/freedesktop/locale1",
+                                  "org.freedesktop.locale1",
+                                  priv->cancellable,
+                                  (GAsyncReadyCallback) localed_proxy_ready,
+                                  self);
+
+        gis_page_set_complete (GIS_PAGE (self), TRUE);
         gtk_widget_show (GTK_WIDGET (self));
 }
 
@@ -179,13 +146,10 @@ gis_keyboard_page_get_help_uri (GisPage *page)
         return "help:gnome-help/prefs-language";
 }
 
-static GtkBuilder *
-gis_keyboard_page_get_builder (GisPage *page)
+static void
+gis_keyboard_page_locale_changed (GisPage *page)
 {
-        GisKeyboardPage *self = GIS_KEYBOARD_PAGE (page);
-        GisKeyboardPagePrivate *priv = self->priv;
-
-        return priv->builder;
+        gis_page_set_title (GIS_PAGE (page), _("Input Sources"));
 }
 
 static void
@@ -197,70 +161,12 @@ gis_keyboard_page_class_init (GisKeyboardPageClass * klass)
 	g_type_class_add_private (klass, sizeof (GisKeyboardPagePrivate));
 
         page_class->page_id = PAGE_ID;
-        page_class->get_builder = gis_keyboard_page_get_builder;
+        page_class->locale_changed = gis_keyboard_page_locale_changed;
 
         object_class->constructed = gis_keyboard_page_constructed;
 	object_class->finalize = gis_keyboard_page_finalize;
 }
 
-static void
-restart_now (GisKeyboardPage *self)
-{
-        GisKeyboardPagePrivate *priv = self->priv;
-
-        gd_notification_dismiss (GD_NOTIFICATION (self->priv->notification));
-
-        g_dbus_proxy_call (priv->session,
-                           "Logout",
-                           g_variant_new ("(u)", 0),
-                           G_DBUS_CALL_FLAGS_NONE,
-                           -1, NULL, NULL, NULL);
-}
-
-static void
-show_restart_notification (GisKeyboardPage *self,
-                           const gchar   *locale)
-{
-	GisKeyboardPagePrivate *priv = self->priv;
-        GtkWidget *box;
-        GtkWidget *label;
-        GtkWidget *button;
-        gchar *current_locale;
-
-        if (priv->notification)
-                return;
-
-        if (locale) {
-                current_locale = g_strdup (setlocale (LC_MESSAGES, NULL));
-                setlocale (LC_MESSAGES, locale);
-        }
-
-        priv->notification = gd_notification_new ();
-        g_object_add_weak_pointer (G_OBJECT (priv->notification),
-                                   (gpointer *)&priv->notification);
-        box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 24);
-        gtk_widget_set_margin_left (box, 12);
-        gtk_widget_set_margin_right (box, 12);
-        gtk_widget_set_margin_top (box, 6);
-        gtk_widget_set_margin_bottom (box, 6);
-        label = gtk_label_new (_("Your session needs to be restarted for changes to take effect"));
-        button = gtk_button_new_with_label (_("Restart Now"));
-        g_signal_connect_swapped (button, "clicked",
-                                  G_CALLBACK (restart_now), self);
-        gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
-        gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
-        gtk_widget_show_all (box);
-
-        gtk_container_add (GTK_CONTAINER (priv->notification), box);
-        gtk_overlay_add_overlay (GTK_OVERLAY (self->priv->overlay), priv->notification);
-        gtk_widget_show (priv->notification);
-
-        if (locale) {
-                setlocale (LC_MESSAGES, current_locale);
-                g_free (current_locale);
-        }
-}
-
 static void
 update_separator_func (GtkWidget **separator,
                        GtkWidget  *child,
@@ -277,188 +183,10 @@ update_separator_func (GtkWidget **separator,
         }
 }
 
-typedef struct {
-        GisKeyboardPage *self;
-        int category;
-        gchar *target_locale;
-} MaybeNotifyData;
-
-static void
-maybe_notify_finish (GObject      *source,
-                     GAsyncResult *res,
-                     gpointer      data)
-{
-        MaybeNotifyData *mnd = data;
-        GisKeyboardPage *self = mnd->self;
-        GError *error = NULL;
-        GVariant *retval = NULL;
-        gchar *current_lang_code = NULL;
-        gchar *current_country_code = NULL;
-        gchar *target_lang_code = NULL;
-        gchar *target_country_code = NULL;
-        const gchar *current_locale = NULL;
-
-        retval = g_dbus_proxy_call_finish (G_DBUS_PROXY (source), res, &error);
-        if (!retval) {
-                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-                        g_warning ("Failed to get locale: %s\n", error->message);
-                goto out;
-        }
-
-        g_variant_get (retval, "(&s)", &current_locale);
-
-        if (!gnome_parse_locale (current_locale,
-                                 &current_lang_code,
-                                 &current_country_code,
-                                 NULL,
-                                 NULL))
-                goto out;
-
-        if (!gnome_parse_locale (mnd->target_locale,
-                                 &target_lang_code,
-                                 &target_country_code,
-                                 NULL,
-                                 NULL))
-                goto out;
-
-        if (g_str_equal (current_lang_code, target_lang_code) == FALSE ||
-            g_str_equal (current_country_code, target_country_code) == FALSE)
-                show_restart_notification (self,
-                                           mnd->category == LC_MESSAGES ? mnd->target_locale : NULL);
-out:
-        g_free (target_country_code);
-        g_free (target_lang_code);
-        g_free (current_country_code);
-        g_free (current_lang_code);
-        g_clear_pointer (&retval, g_variant_unref);
-        g_clear_error (&error);
-        g_free (mnd->target_locale);
-        g_free (mnd);
-}
-
-static void
-maybe_notify (GisKeyboardPage *self,
-              int            category,
-              const gchar   *target_locale)
-{
-        GisKeyboardPagePrivate *priv = self->priv;
-        MaybeNotifyData *mnd;
-
-        mnd = g_new0 (MaybeNotifyData, 1);
-        mnd->self = self;
-        mnd->category = category;
-        mnd->target_locale = g_strdup (target_locale);
-
-        g_dbus_proxy_call (priv->session,
-                           "GetLocale",
-                           g_variant_new ("(i)", category),
-                           G_DBUS_CALL_FLAGS_NONE,
-                           -1,
-                           priv->cancellable,
-                           maybe_notify_finish,
-                           mnd);
-}
-
-static void set_localed_locale (GisKeyboardPage *self);
-
-static void
-update_region (GisKeyboardPage *self,
-               const gchar   *region)
-{
-	GisKeyboardPagePrivate *priv = self->priv;
-
-        if (g_strcmp0 (region, priv->region) == 0)
-                return;
-
-        g_settings_set_string (priv->locale_settings, KEY_REGION, region);
-        maybe_notify (self, LC_TIME, region);
-}
 
 static void show_input_chooser (GisKeyboardPage *self);
 static void remove_selected_input (GisKeyboardPage *self);
 
-static void
-permission_acquired (GObject      *source,
-                     GAsyncResult *res,
-                     gpointer      data)
-{
-        GisKeyboardPage *self = data;
-	GisKeyboardPagePrivate *priv = self->priv;
-        GError *error = NULL;
-        gboolean allowed;
-
-        allowed = g_permission_acquire_finish (priv->permission, res, &error);
-        if (error) {
-                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-                        g_warning ("Failed to acquire permission: %s\n", error->message);
-                g_error_free (error);
-                return;
-        }
-
-        if (allowed) {
-                switch (priv->op) {
-                case ADD_INPUT:
-                        show_input_chooser (self);
-                        break;
-                case REMOVE_INPUT:
-                        remove_selected_input (self);
-                        break;
-                default:
-                        g_warning ("Unknown privileged operation: %d\n", priv->op);
-                        break;
-                }
-        }
-}
-
-static void
-update_region_label (GisKeyboardPage *self)
-{
-        GisKeyboardPagePrivate *priv = self->priv;
-        const gchar *region;
-        gchar *name;
-
-        if (priv->region == NULL || priv->region[0] == '\0')
-                region = priv->language;
-        else
-                region = priv->region;
-
-        name = gnome_get_country_from_locale (region, region);
-        gtk_label_set_label (GTK_LABEL (priv->formats_label), name);
-        g_free (name);
-}
-
-static void
-update_region_from_setting (GisKeyboardPage *self)
-{
-        GisKeyboardPagePrivate *priv = self->priv;
-
-        g_free (priv->region);
-        priv->region = g_settings_get_string (priv->locale_settings, KEY_REGION);
-        update_region_label (self);
-}
-
-static void
-update_language_label (GisKeyboardPage *self)
-{
-	GisKeyboardPagePrivate *priv = self->priv;
-        const gchar *language;
-        gchar *name;
-
-        if (priv->login)
-                language = priv->system_language;
-        else
-                language = priv->language;
-        if (language)
-                name = gnome_get_language_from_locale (language, language);
-        else
-                name = g_strdup (C_("Language", "None"));
-        gtk_label_set_label (GTK_LABEL (priv->language_label), name);
-        g_free (name);
-
-        /* Formats will change too if not explicitly set. */
-        update_region_label (self);
-}
-
 #ifdef HAVE_IBUS
 static void
 update_ibus_active_sources (GisKeyboardPage *self)
@@ -625,15 +353,6 @@ add_input_row (GisKeyboardPage   *self,
         GtkWidget *label;
         GtkWidget *image;
 
-        if (priv->login) {
-                GList *l;
-                l = gtk_container_get_children (GTK_CONTAINER (priv->input_list));
-                if (l && l->next == NULL &&
-                    g_strcmp0 (g_object_get_data (G_OBJECT (l->data), "type"), "none") == 0)
-                        gtk_container_remove (GTK_CONTAINER (priv->input_list), GTK_WIDGET (l->data));
-                g_list_free (l);
-        }
-
         row = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
         label = gtk_label_new (name);
         gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
@@ -670,51 +389,59 @@ add_input_row (GisKeyboardPage   *self,
 }
 
 static void
-add_input_sources (GisKeyboardPage *self,
-                   GVariant      *sources)
+add_input_source (GisKeyboardPage *self,
+                  const gchar     *type,
+                  const gchar     *id)
 {
         GisKeyboardPagePrivate *priv = self->priv;
-        GVariantIter iter;
-        const gchar *type;
-        const gchar *id;
         const gchar *name;
         gchar *display_name;
         GDesktopAppInfo *app_info;
 
-        g_variant_iter_init (&iter, sources);
-        while (g_variant_iter_next (&iter, "(&s&s)", &type, &id)) {
-                display_name = NULL;
-                app_info = NULL;
+        display_name = NULL;
+        app_info = NULL;
 
-                if (g_str_equal (type, INPUT_SOURCE_TYPE_XKB)) {
-                        gnome_xkb_info_get_layout_info (priv->xkb_info, id, &name, NULL, NULL, NULL);
-                        if (!name) {
-                                g_warning ("Couldn't find XKB input source '%s'", id);
-                                continue;
-                        }
-                        display_name = g_strdup (name);
-                        type = INPUT_SOURCE_TYPE_XKB;
-#ifdef HAVE_IBUS
-                } else if (g_str_equal (type, INPUT_SOURCE_TYPE_IBUS)) {
-                        IBusEngineDesc *engine_desc = NULL;
-
-                        if (priv->ibus_engines)
-                                engine_desc = g_hash_table_lookup (priv->ibus_engines, id);
-                        if (engine_desc)
-                                display_name = engine_get_display_name (engine_desc);
-
-                        app_info = setup_app_info_for_id (id);
-                        type = INPUT_SOURCE_TYPE_IBUS;
-#endif
-                } else {
-                        g_warning ("Unhandled input source type '%s'", type);
-                        continue;
+        if (g_str_equal (type, INPUT_SOURCE_TYPE_XKB)) {
+                gnome_xkb_info_get_layout_info (priv->xkb_info, id, &name, NULL, NULL, NULL);
+                if (!name) {
+                        g_warning ("Couldn't find XKB input source '%s'", id);
+                        return;
                 }
+                display_name = g_strdup (name);
+                type = INPUT_SOURCE_TYPE_XKB;
+#ifdef HAVE_IBUS
+        } else if (g_str_equal (type, INPUT_SOURCE_TYPE_IBUS)) {
+                IBusEngineDesc *engine_desc = NULL;
 
-                add_input_row (self, type, id, display_name ? display_name : id, app_info);
-                g_free (display_name);
-                g_clear_object (&app_info);
+                if (priv->ibus_engines)
+                        engine_desc = g_hash_table_lookup (priv->ibus_engines, id);
+                if (engine_desc)
+                        display_name = engine_get_display_name (engine_desc);
+
+                app_info = setup_app_info_for_id (id);
+                type = INPUT_SOURCE_TYPE_IBUS;
+#endif
+        } else {
+                g_warning ("Unhandled input source type '%s'", type);
+                return;
         }
+
+        add_input_row (self, type, id, display_name ? display_name : id, app_info);
+        g_free (display_name);
+        g_clear_object (&app_info);
+}
+
+static void
+add_input_sources (GisKeyboardPage *self,
+                   GVariant        *sources)
+{
+        GVariantIter iter;
+        const gchar *type;
+        const gchar *id;
+
+        g_variant_iter_init (&iter, sources);
+        while (g_variant_iter_next (&iter, "(&s&s)", &type, &id))
+                add_input_source (self, type, id);
 }
 
 static void
@@ -865,38 +592,7 @@ static void set_localed_input (GisKeyboardPage *self);
 static void
 update_input (GisKeyboardPage *self)
 {
-	GisKeyboardPagePrivate *priv = self->priv;
-
-        if (priv->login) {
-                set_localed_input (self);
-        } else {
-                set_input_settings (self);
-        }
-}
-
-static void
-apologize_for_no_ibus_login (GisKeyboardPage *self)
-{
-        GtkWidget *dialog;
-        GtkWidget *toplevel;
-        GtkWidget *image;
-
-        toplevel = gtk_widget_get_toplevel (GTK_WIDGET (self));
-
-        dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-                                         GTK_DIALOG_MODAL,
-                                         GTK_MESSAGE_OTHER,
-                                         GTK_BUTTONS_OK,
-                                         _("Sorry"));
-        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                                  "%s", _("Input methods can't be used on the login screen"));
-        image = gtk_image_new_from_icon_name ("face-sad-symbolic",
-                                              GTK_ICON_SIZE_DIALOG);
-        gtk_widget_show (image);
-        gtk_message_dialog_set_image (GTK_MESSAGE_DIALOG (dialog), image);
-
-        gtk_dialog_run (GTK_DIALOG (dialog));
-        gtk_widget_destroy (dialog);
+        set_input_settings (self);
 }
 
 static gboolean
@@ -942,13 +638,10 @@ input_response (GtkWidget *chooser, gint response_id, gpointer data)
                                 type = INPUT_SOURCE_TYPE_XKB;
                         }
 
-                        if (priv->login && g_str_equal (type, INPUT_SOURCE_TYPE_IBUS)) {
-                                apologize_for_no_ibus_login (self);
-                        } else {
-                                add_input_row (self, type, id, name, app_info);
-                                update_buttons (self);
-                                update_input (self);
-                        }
+                        add_input_row (self, type, id, name, app_info);
+                        update_buttons (self);
+                        update_input (self);
+
                         g_free (id);
                         g_free (name);
                         g_clear_object (&app_info);
@@ -984,19 +677,7 @@ show_input_chooser (GisKeyboardPage *self)
 static void
 add_input (GisKeyboardPage *self)
 {
-	GisKeyboardPagePrivate *priv = self->priv;
-
-        if (!priv->login) {
-                show_input_chooser (self);
-        } else if (g_permission_get_allowed (priv->permission)) {
-                show_input_chooser (self);
-        } else if (g_permission_get_can_acquire (priv->permission)) {
-                priv->op = ADD_INPUT;
-                g_permission_acquire_async (priv->permission,
-                                            NULL,
-                                            permission_acquired,
-                                            self);
-        }
+        show_input_chooser (self);
 }
 
 static GtkWidget *
@@ -1054,19 +735,7 @@ do_remove_selected_input (GisKeyboardPage *self)
 static void
 remove_selected_input (GisKeyboardPage *self)
 {
-	GisKeyboardPagePrivate *priv = self->priv;
-
-        if (!priv->login) {
-                do_remove_selected_input (self);
-        } else if (g_permission_get_allowed (priv->permission)) {
-                do_remove_selected_input (self);
-        } else if (g_permission_get_can_acquire (priv->permission)) {
-                priv->op = REMOVE_INPUT;
-                g_permission_acquire_async (priv->permission,
-                                            NULL,
-                                            permission_acquired,
-                                            self);
-        }
+        do_remove_selected_input (self);
 }
 
 static void
@@ -1160,6 +829,22 @@ show_selected_layout (GisKeyboardPage *self)
         g_free (commandline);
 }
 
+static void
+add_default_input_source_for_locale (GisKeyboardPage *self)
+{
+        GisKeyboardPagePrivate *priv = self->priv;
+        const gchar *locale;
+        const gchar *type;
+        const gchar *id;
+
+        locale = gis_driver_get_user_language (GIS_PAGE (self)->driver);
+
+        if (!gnome_get_input_source_from_locale (locale, &type, &id))
+                return;
+
+        add_input_source (self, type, id);
+}
+
 static void
 setup_input_section (GisKeyboardPage *self)
 {
@@ -1211,55 +896,7 @@ setup_input_section (GisKeyboardPage *self)
         g_signal_connect (priv->input_settings, "changed::" KEY_INPUT_SOURCES,
                           G_CALLBACK (input_sources_changed), self);
 
-        add_input_sources_from_settings (self);
-        update_buttons (self);
-}
-
-static void
-on_localed_properties_changed (GDBusProxy     *proxy,
-                               GVariant       *changed_properties,
-                               const gchar   **invalidated_properties,
-                               GisKeyboardPage  *self)
-{
-	GisKeyboardPagePrivate *priv = self->priv;
-        GVariant *v;
-
-        v = g_dbus_proxy_get_cached_property (proxy, "Locale");
-        if (v) {
-                const gchar **strv;
-                gsize len;
-                gint i;
-                const gchar *lang, *messages, *time;
-
-                strv = g_variant_get_strv (v, &len);
-
-                lang = messages = time = NULL;
-                for (i = 0; strv[i]; i++) {
-                        if (g_str_has_prefix (strv[i], "LANG=")) {
-                                lang = strv[i] + strlen ("LANG=");
-                        } else if (g_str_has_prefix (strv[i], "LC_MESSAGES=")) {
-                                messages = strv[i] + strlen ("LC_MESSAGES=");
-                        } else if (g_str_has_prefix (strv[i], "LC_TIME=")) {
-                                time = strv[i] + strlen ("LC_TIME=");
-                        }
-                }
-                if (!lang) {
-                        lang = "";
-                }
-                if (!messages) {
-                        messages = lang;
-                }
-                if (!time) {
-                        time = lang;
-                }
-                g_free (priv->system_language);
-                priv->system_language = g_strdup (messages);
-                g_free (priv->system_region);
-                priv->system_region = g_strdup (time);
-                g_variant_unref (v);
-
-                update_language_label (self);
-        }
+        add_default_input_source_for_locale (self);
 }
 
 static void
@@ -1306,57 +943,18 @@ add_input_sources_from_localed (GisKeyboardPage *self)
                 else
                         id = g_strdup (layouts[i]);
 
-                gnome_xkb_info_get_layout_info (priv->xkb_info, id, &name, NULL, NULL, NULL);
-
-                add_input_row (self, INPUT_SOURCE_TYPE_XKB, id, name ? name : id, NULL);
+                if (!input_source_already_added (self, id)) {
+                        gnome_xkb_info_get_layout_info (priv->xkb_info, id, &name, NULL, NULL, NULL);
+                        add_input_row (self, INPUT_SOURCE_TYPE_XKB, id, name ? name : id, NULL);
+                }
 
                 g_free (id);
         }
-        if (n == 0) {
-                add_input_row (self, "none", "none", _("No input source selected"), NULL);
-        }
 
         g_strfreev (variants);
         g_strfreev (layouts);
 }
 
-static void
-set_localed_locale (GisKeyboardPage *self)
-{
-	GisKeyboardPagePrivate *priv = self->priv;
-        GVariantBuilder *b;
-        gchar *s;
-
-        b = g_variant_builder_new (G_VARIANT_TYPE ("as"));
-        s = g_strconcat ("LANG=", priv->system_language, NULL);
-        g_variant_builder_add (b, "s", s);
-        g_free (s);
-
-        if (g_strcmp0 (priv->system_language, priv->system_region) != 0) {
-                s = g_strconcat ("LC_TIME=", priv->system_region, NULL);
-                g_variant_builder_add (b, "s", s);
-                g_free (s);
-                s = g_strconcat ("LC_NUMERIC=", priv->system_region, NULL);
-                g_variant_builder_add (b, "s", s);
-                g_free (s);
-                s = g_strconcat ("LC_MONETARY=", priv->system_region, NULL);
-                g_variant_builder_add (b, "s", s);
-                g_free (s);
-                s = g_strconcat ("LC_MEASUREMENT=", priv->system_region, NULL);
-                g_variant_builder_add (b, "s", s);
-                g_free (s);
-                s = g_strconcat ("LC_PAPER=", priv->system_region, NULL);
-                g_variant_builder_add (b, "s", s);
-                g_free (s);
-        }
-        g_dbus_proxy_call (priv->localed,
-                           "SetLocale",
-                           g_variant_new ("(asb)", b, TRUE),
-                           G_DBUS_CALL_FLAGS_NONE,
-                           -1, NULL, NULL, NULL);
-        g_variant_builder_unref (b);
-}
-
 static void
 set_localed_input (GisKeyboardPage *self)
 {
@@ -1420,101 +1018,15 @@ localed_proxy_ready (GObject      *source,
         priv = self->priv;
         priv->localed = proxy;
 
-        gtk_widget_set_sensitive (priv->login_button, TRUE);
-
-        g_signal_connect (priv->localed, "g-properties-changed",
-                          G_CALLBACK (on_localed_properties_changed), self);
-        on_localed_properties_changed (priv->localed, NULL, NULL, self);
-}
-
-static void
-login_changed (GisKeyboardPage *self)
-{
-	GisKeyboardPagePrivate *priv = self->priv;
-        gboolean can_acquire;
-
-        priv->login = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->login_button));
-        gtk_widget_set_visible (priv->formats_row, !priv->login);
-        gtk_widget_set_visible (priv->login_label, priv->login);
-
-        can_acquire = priv->permission &&
-                (g_permission_get_allowed (priv->permission) ||
-                 g_permission_get_can_acquire (priv->permission));
-        /* FIXME: insensitive doesn't look quite right for this */
-        gtk_widget_set_sensitive (priv->language_section, !priv->login || can_acquire);
-        gtk_widget_set_sensitive (priv->input_section, !priv->login || can_acquire);
-
-        clear_input_sources (self);
-        if (priv->login)
-                add_input_sources_from_localed (self);
-        else
-                add_input_sources_from_settings (self);
-
-        update_language_label (self);
+        add_input_sources_from_localed (self);
         update_buttons (self);
 }
 
-
-static void
-session_proxy_ready (GObject      *source,
-                     GAsyncResult *res,
-                     gpointer      data)
-{
-        GisKeyboardPage *self = data;
-        GDBusProxy *proxy;
-        GError *error = NULL;
-
-        proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
-
-        if (!proxy) {
-                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-                        g_warning ("Failed to contact gnome-session: %s\n", error->message);
-                g_error_free (error);
-                return;
-        }
-
-        self->priv->session = proxy;
-}
-
 static void
 gis_keyboard_page_init (GisKeyboardPage *self)
 {
-	GisKeyboardPagePrivate *priv;
-	GError *error = NULL;
-
-	priv = self->priv = KEYBOARD_PAGE_PRIVATE (self);
+        self->priv = KEYBOARD_PAGE_PRIVATE (self);
         g_resources_register (keyboard_get_resource ());
-
-	priv->builder = gtk_builder_new ();
-
-	gtk_builder_add_from_resource (priv->builder,
-                                       "/org/gnome/initial-setup/gis-keyboard-page.ui",
-                                       &error);
-	if (error != NULL) {
-		g_warning ("Error loading UI file: %s", error->message);
-		g_error_free (error);
-		return;
-	}
-
-        priv->user_manager = act_user_manager_get_default ();
-
-        priv->cancellable = g_cancellable_new ();
-
-        g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
-                                  G_DBUS_PROXY_FLAGS_NONE,
-                                  NULL,
-                                  "org.gnome.SessionManager",
-                                  "/org/gnome/SessionManager",
-                                  "org.gnome.SessionManager",
-                                  priv->cancellable,
-                                  session_proxy_ready,
-                                  self);
-
-        setup_input_section (self);
-
-        priv->overlay = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-                                                            "keyboard_page"));
-        gtk_container_add (GTK_CONTAINER (self), priv->overlay);
 }
 
 void
